@@ -9,10 +9,20 @@ do we need all these functions as static?
 some may be instance functions, some may belong in utils.
 """
 
-import numpy as np
+import warnings
 import copy
 
-from gensound.utils import sec
+import numpy as np
+
+from gensound.utils import sec, sample_rates
+
+
+def warning_msg(message, category, filename, lineno, file=None, line=None):
+    return '%s: %s\n' % (category.__name__, message)
+
+warnings.formatwarning = warning_msg
+
+
 
 class Audio:
     """Basically a wrapper for a numpy array, representing the signal.
@@ -175,12 +185,16 @@ class Audio:
         Don't use this directly; better use Raw().resample instead.
         [TODO does this have to be a supported sample rate?]
         """
+        if sample_rate not in sample_rates:
+            warnings.warn("Resampling audio to sample rate which is not supported in playback. "
+                          "This is totally fine if you know what you're doing.")
+        
         if self.sample_rate == sample_rate:
             return
         from gensound.utils import get_interpolation
         interpolate = get_interpolation(method)
         factor = self.sample_rate / sample_rate
-        self.audio = interpolate(self.audio, np.arange(0, self.length-1, factor))
+        self.audio = interpolate(self.audio, np.arange(0, self.length-1, factor)) # TODO should it be self.length - 1? seems to say arange gives half-open range...
         self.sample_rate = sample_rate
     
     ### channel manipulations ###
@@ -260,7 +274,35 @@ class Audio:
     
     
     
+    #### test #####
+    # these are overloaded for two reasons:
+    # 1. brevity (now transforms can modify audio instead of audio.audio; be careful not to lose references though)
+    # 2. (more importantly) convenient syntax to access float indices of the audio!
+    # the floats handled below are sample numbers which are interpolated.
+    def __getitem__(self, key):
+        assert len(key) == 2, "Audio objects must be doubly-subscripted for channel and samples"
+        samples_key = key[1]
+        
+        if isinstance(samples_key, slice) and isinstance(samples_key.start, float):
+            raise NotImplementedError("Gensound decided against slicing audio with floats because it thought nobody would use that. Let us know?")
+            # slice of floats
+            #from gensound.utils import get_interpolation
+            #interpolate = get_interpolation('quadratic') # TODO use gensound settings for default
+            #return interpolate(self.audio[key[0],:], np.arange(samples_key.start or 0.0,
+            #                                                   samples_key.stop or self.length - 0.0,
+            #                                                   samples_key.step or 1.0))
+        
+        if isinstance(samples_key, (list, tuple, np.ndarray)) and isinstance(samples_key[0], float):
+            # iterable of floats
+            from gensound.utils import get_interpolation
+            interpolate = get_interpolation('quadratic') # TODO use gensound settings for default
+            return interpolate(self.audio[key[0],:], samples_key)
+            
+        
+        return self.audio.__getitem__(key)
     
+    def __setitem__(self, key, value):
+        return self.audio.__setitem__(key, value)
     
     
     
@@ -280,31 +322,50 @@ class Audio:
     But export should actually be called from Signal, and do the rest on its own.
     """
     
-    def fit(self, max_amplitude=1):
+    def fit(self, max_amplitude):
         """
-        stretches/squashes the amplitude of the samples to be [-max_amplitude, +max_amplitude]
-        for max_amplitude <= 1.
+        If max_amplitude = None (default), ensure the signal lies within [-1,1],
+          shrinking it if necessary (with a warning).
         
-        for max_amplitude = None, leave things as they are.
+        If max_amplitude is a positive number, in which case ensure the signal lies within
+          [-max_amplitude, +max_amplitude].
         
-        'destructive' function, called only when exporting or playing.s
+        Finally, if max_amplitude = 0, don't touch anything.
+        
+        TODO do we need all 3 situations? is this the best interface for them?
+        
+        'destructive' function, called only when exporting or playing.
         """
-        assert max_amplitude is None or 0 < max_amplitude
+        assert max_amplitude is None or 0 <= max_amplitude
         
-        if max_amplitude == None:
+        if max_amplitude == 0:
+            warnings.warn("Setting max_amplitude to 0 may result in distorted audio or clipping.")
+            # leave signal peaks unchanged
             return
         
+        if max_amplitude is not None and max_amplitude > 1:
+            warnings.warn("Supplying a value greater than 1 for max_amplitude will likely lead to distorted audio and clipping.")
+        
         max_amp = np.max(np.abs(self.audio))
         
-        if max_amp == 0:
+        if max_amplitude is None and max_amp <= 1:
+            # fits within default range
             return
         
+        # shrink/stretch to either max_amplitude or to 1
+        if max_amplitude is None:
+            warnings.warn(f"Output audio signal amplitude exceeds 1 (max abs. amplitude {max_amp:0.2f}). "
+                          "By default, the signal will be shrunk to fit range of [-1, 1] to prevent clipping. "
+                          "To prevent this behaviour, set the max_amplitude argument to zero, which leaves the signal untouched, "
+                          "or to any positive number, which will stretch/shrink it to match the given peak amplitude.")
+            max_amplitude = 1
+        
         self.audio = self.audio * (max_amplitude / max_amp)
         
     
     ####### post-mixdown ########
     
-    def _prepare_buffer(self, byte_width=2, max_amplitude=1):
+    def _prepare_buffer(self, byte_width, max_amplitude):
         """ Attaches byte width to Audio object, and converts audio information
         to bytes object. This is only done in preparation for output (file or playback).
         """
@@ -317,22 +378,68 @@ class Audio:
         self.buffer = audio_to_bytes(self.audio, ["uint8","int16","int24","int32"][byte_width-1])
         
         
-    def play(self, byte_width=2, is_wait=True):
+    def play(self, byte_width=2, max_amplitude=None, is_wait=True):
         from gensound.io import play_Audio
         
-        self._prepare_buffer(byte_width) # TODO max amplitude
+        self._prepare_buffer(byte_width, max_amplitude) # TODO max amplitude
         play_Audio(self, is_wait)
     
+    @staticmethod
+    def from_WAV(filename):
+        warnings.warn("Audio.from_WAV to be deprecated; use Audio.from_file instead.")
+        return Audio.from_file(filename, file_format="wav")
+    
+    @staticmethod
+    def from_file(filename, file_format=None): # 2nd argument to force format regardless of file name
+        ext = file_format or filename.split(".")[-1].lower()
+        
+        if ext in ("wav", "wave"):
+            from gensound.io import WAV_to_Audio
+            return WAV_to_Audio(filename)
+        
+        if ext in ("aiff", "aifc", "aif"):
+            from gensound.io import AIFF_to_Audio
+            return AIFF_to_Audio(filename)
+    
     def to_WAV(self, filename, byte_width=2, max_amplitude=1):
-        from gensound.io import export_WAV
+        warnings.warn("Audio.to_WAV to be deprecated; use Audio.export instead.")
+        self.export(filename, byte_width, max_amplitude, file_format="wav")
+    
+    def export(self, filename, byte_width=2, max_amplitude=None, file_format=None):
+        ext = file_format or filename.split(".")[-1].lower()
         
         self._prepare_buffer(byte_width, max_amplitude)
         
         # TODO disentangle export_WAV arguments so that they will be given
         # seperately (or in a config dict) rather than in an Audio object,
         # since the latter is not supposed to hold a buffer nor byte width
-        export_WAV(filename, self)
-    
+        
+        if ext in ("wav", "wave"):
+            from gensound.io import export_WAV
+            export_WAV(filename, self)
+        
+        if ext in ("aiff", "aifc", "aif"):
+            from gensound.io import export_AIFF
+            export_AIFF(filename, self)
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
