@@ -19,6 +19,11 @@ There is a class for each supported I/O library, which provides these functions
 for that library in particular. When this file is loaded, the desired libraries
 are determined from settings.py (TODO).
 
+
+Note: The names defined below should only be imported in audio.py.
+The Audio class is the final intermediary between Gensound and the actual IO.
+
+
 """
 
 import wave
@@ -45,6 +50,42 @@ class _IO_wave:
     
         file.writeframes(audio.buffer)
 
+class _IO_aifc:
+    is_supported = True # python built-in
+    
+    @staticmethod
+    def export_AIFF(filename, audio):
+        import aifc
+        
+        with aifc.open(filename, "wb") as file:
+            file.setnchannels(audio.num_channels)
+            file.setsampwidth(audio.byte_width)
+            file.setframerate(audio.sample_rate)
+            file.setnframes(audio.length)
+            
+            file.writeframes(audio.buffer.byteswap()) # AIFF byteswap
+    
+    @staticmethod
+    def AIFF_to_Audio(filename):
+        # when making changes, consider WAV_to_Audio as well
+        import aifc
+        
+        with aifc.open(filename, "rb") as file:
+            buffer = np.frombuffer(file.readframes(file.getnframes()),
+                                   [np.uint8,np.int16,None,np.int32][file.getsampwidth()-1])
+            buffer = buffer.byteswap().astype(np.float64)
+            
+            buffer = np.reshape(buffer,
+                                newshape=(file.getnchannels(),
+                                          int(len(buffer)/file.getnchannels()))[::-1]).T.copy(order='C')
+            buffer /= 2**(8*file.getsampwidth()-1)
+            
+            from gensound.audio import Audio
+            audio = Audio(file.getframerate())
+            audio.from_array(buffer)
+            
+        return audio
+
 class _IO_SA:
     is_supported = "simpleaudio" in _supported
     
@@ -61,8 +102,31 @@ class _IO_SA:
             play_obj.stop()
         else:
             play_obj.wait_done()
+    
+    @staticmethod
+    def WAV_to_Audio(filename=""):
+        wav = sa.WaveObject.from_wave_file(filename)
+        # TODO support int24
+        buffer = np.frombuffer(wav.audio_data, [np.uint8,np.int16,None,np.int32][wav.bytes_per_sample-1]).astype(np.float64)
+        
+        buffer = np.reshape(buffer,
+                            newshape=(wav.num_channels,
+                                      int(len(buffer)/wav.num_channels))[::-1]).T.copy(order='C')
+        buffer /= 2**(8*wav.bytes_per_sample-1)
+        # TODO the above needs some consideration
+        # should we convert to float immediately, or wait till the last minute?
+        # is this the right place to convert?
+        # and should this be normalized in some way in relation to the synthesized signals?
+        from gensound.audio import Audio
+        audio = Audio(wav.sample_rate)
+        audio.from_array(buffer)
+        
+        return audio
 
 
+
+### for testing purposes; outputs WAV and logs the cade that generated it ####
+
 def export_test(audio, func=None, byte_width=2, max_amplitude=1):
     if not hasattr(audio, "buffer"):
         audio._prepare_buffer(byte_width, max_amplitude)
@@ -85,39 +149,7 @@ def export_test(audio, func=None, byte_width=2, max_amplitude=1):
                                                                timestamp,
                                                                inspect.getsource(func)))
 
-def play_WAV(filename="", is_wait=False):
-    wav = sa.WaveObject.from_wave_file(filename)
-    play = wav.play()
-    
-    if not is_wait:
-        input("Type something to quit.")
-        play.stop()
-    else:
-        play.wait_done()
-
-
-def WAV_to_Audio(filename=""):
-    wav = sa.WaveObject.from_wave_file(filename)
-    # TODO support int24
-    buffer = np.frombuffer(wav.audio_data, [np.uint8,np.int16,None,np.int32][wav.bytes_per_sample-1]).astype(np.float64)
-    
-    buffer = np.reshape(buffer,
-                        newshape=(wav.num_channels,
-                                  int(len(buffer)/wav.num_channels))[::-1]).T.copy(order='C')
-    buffer /= 2**(8*wav.bytes_per_sample-1)
-    # TODO the above needs some consideration
-    # should we convert to float immediately, or wait till the last minute?
-    # is this the right place to convert?
-    # and should this be normalized in some way in relation to the synthesized signals?
-    from gensound.audio import Audio
-    audio = Audio(wav.sample_rate)
-    audio.from_array(buffer)
-    
-    return audio
-
-
-
-###################
+#### EXPORTED NAMES ##########
 '''
 Here are the exported functionalities.
 TODO add logic to figure out supported libraries (also let user customize),
@@ -125,10 +157,13 @@ and direct traffic to the correct classes.
 also figure out how to navigate different formats
 '''
 
-export_WAV = _IO_wave.export_WAV
 play_Audio = _IO_SA.playback # should be named 'playback' anywhere but in Audio
 
+export_WAV = _IO_wave.export_WAV
+WAV_to_Audio = _IO_SA.WAV_to_Audio
 
+export_AIFF = _IO_aifc.export_AIFF
+AIFF_to_Audio = _IO_aifc.AIFF_to_Audio
 
 
 
