@@ -15,7 +15,6 @@ from gensound.musicTheory import parse_melody_to_signal, read_freq
 from gensound.transforms import Transform, TransformChain, Amplitude, Slice, Combine, BiTransform
 from gensound.curve import Curve
 from gensound.audio import Audio
-from gensound.io import WAV_to_Audio
 
 #CHANNEL_NAMES = {"L":0, "R":1}
 
@@ -56,21 +55,18 @@ class Signal:
     
     mixdown = realise # TODO consider keeping just 1
     
-    def play(self, sample_rate=44100, byte_width=2, max_amplitude=1):
+    def play(self, sample_rate=44100, **kwargs):
         audio = self.realise(sample_rate)
-        audio.play(byte_width)
+        audio.play(**kwargs)
     
-    def export(self, filename, sample_rate=44100, byte_width=2, max_amplitude=1):
-        """
-        0 < max_amplitude <= 1 implies stretching the amplitudes
-        so they would hit absolute value of max_amplitude.
-        otherwise, max_amplitude = None implies not to touch the amplitudes
-        as given, unless they exceed 1 in which case we shrink everything proportionally.
-        TODO not sure this behaviour is optimal
-        """
+    def export(self, filename, sample_rate=44100, **kwargs):
         audio = self.realise(sample_rate)
-        audio.to_WAV(filename, byte_width, max_amplitude=max_amplitude)
-        
+        audio.export(filename, **kwargs)
+    
+    def to_bytes(self, sample_rate=44100, **kwargs):
+        audio = self.realise(sample_rate)
+        audio._prepare_buffer(**kwargs)
+        return audio.buffer
     
     #####################
     def num_samples(self, sample_rate):
@@ -502,13 +498,13 @@ class Sine(Oscillator): # oscillator? pitch? phaser?
     wave = np.sin
 
 class Triangle(Oscillator): # TODO start at 0, not 1
-    wave = lambda phase: 2*np.abs((phase % (2*np.pi) - np.pi))/np.pi - 1
+    wave = lambda phase: 2*np.abs(((phase-0.5*np.pi) % (2*np.pi) - np.pi))/np.pi - 1
     
 class Square(Oscillator):
     wave = lambda phase: ((phase % (2*np.pi) < np.pi)*2 - 1).astype(np.float64)
 
 class Sawtooth(Oscillator):
-    wave = lambda phase: (phase % (2*np.pi))/np.pi-1
+    wave = lambda phase: ((phase+np.pi) % (2*np.pi))/np.pi-1
 
 # TODO sweepsine, periodic impulse
 
@@ -526,7 +522,7 @@ class Raw(Signal):
         
         if audio != None:
             if not hasattr(self, "key"):
-                self.key = len(Raw.cache)
+                self.key = f"Raw_{len(Raw.cache)}"
             
             if self._key() not in Raw.cache:
                 Raw.cache[self._key()] = audio
@@ -535,6 +531,11 @@ class Raw(Signal):
         return type(self).__name__ + ":" + str(self.key)
     
     def resample(self, sample_rate=44100, method="quadratic"):
+        # TODO maybe copy resampled audio under new key which indicates the sample rate change
+        # but to do that we need to decide if we clear the previous version from cache,
+        # or what is the best way to let the user decide (e.g. what should be the default behavior?)
+        # another approach is to add "new_copy" arg to WAV, ensuring it gets a fresh key
+        # and reloads the WAV
         self.audio._resample(sample_rate, method)
         return self # so the user can type WAV("a.wav").resample(44100)*Transform() etc.
         ...
@@ -574,12 +575,15 @@ class WAV(Raw):
     """
     
     def __init__(self, filename):
-        self.key = filename
+        # TODO consider adding "new_copy"/"force_copy"/"unique"/"new" boolean arg
+        # would be implemented either by appending random nonce to key,
+        # or adding counter for number of copies of base key
+        self.key = f"WAV_{filename}"
         
         audio = None
         
         if self._key() not in Raw.cache:
-            audio = WAV_to_Audio(filename)
+            audio = Audio.from_file(filename)
         
         # TODO copy again? so the cache will be eternally independent?
         super().__init__(audio)
